////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// 
///  Copyright 2010 Aurora Feint, Inc.
/// 
///  Licensed under the Apache License, Version 2.0 (the "License");
///  you may not use this file except in compliance with the License.
///  You may obtain a copy of the License at
///  
///      http://www.apache.org/licenses/LICENSE-2.0
///      
///  Unless required by applicable law or agreed to in writing, software
///  distributed under the License is distributed on an "AS IS" BASIS,
///  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
///  See the License for the specific language governing permissions and
///  limitations under the License.
/// 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
//  AppController+OpenFeint.mm
//  Unity-iPhone
//
// This code was contributed to OpenFeint by Tiny Tim Games (www.tinytimgames.com).
// It is thanks to awesome developers like these that OpenFeint can keep rocking!
//

#import <UIKit/UIKit.h>
#import "AppController.h"
#import "OpenFeintSettingsDefines.h"

#import "OpenFeint.h"
#import "OpenFeint+Dashboard.h"
#import "OpenFeint+UserOptions.h"
#import "OFHighScoreService.h"
#import "OFAchievementService.h"
#import "OFSocialNotificationService.h"
#import "OFFriendsService.h"
#import "OFChallengeService.h"
#import "OFChallenge.h"
#import "OFChallengeDefinition.h"
#import "OFChallengeDefinitionService.h"
#import "OFChallengeToUser.h"
#import "OFCloudStorageService.h"

// Defines.
#define OF_CMD(x)		[OFCommand isEqualToString:x]
#define OF_METHOD(x)	mono_get_method_from_image(x, monoImage)

// Enumerations.
// Matches ePage enumeration in OpenFeint.cs
enum ePage
{
	ePageMain,
	ePageLeaderboards,
	ePageAchievements,
	ePageChallenges,
	ePageFindFriends,
	ePageWhosPlaying,
	ePageGlobalChatRooms,
	ePageSwitchUser,
};

// Matches indices of private array generalEvents in OpenFeint.cs
enum eGeneralEvent
{
	eGeneralEventOnRestartedChallenge,
	eGeneralEventOnRestartedCreateChallenge,
	eGeneralEventOnCompletedChallengeScreenClosed,
	eGeneralEventOnSendChallengeScreenClosed,
	eGeneralEventOnUserSentChallenges,
};

// Matches indices of private array deferredResultEvents in OpenFeint.cs
enum eDeferredResultEvent
{
	eDeferredResultEventOnHighScoreSubmitted,
	eDeferredResultEventOnAchievementUnlocked,
	eDeferredResultEventOnChallengeResultSubmitted,
	eDeferredResultEventOnUploadedCloudBlob,
	eDeferredResultEventOnDownloadedCloudBlob,
};

// Matches indices of private array userLogInEvents in OpenFeint.cs
enum eUserLogInEvent
{
	eUserLogInEventOnUserLoggedIn,
	eUserLogInEventOnUserLoggedOut,
};

// Constants.
static const NSString *ofCommandKey = @"OFCommand";
static const NSString *ofResultKey = @"OFResult";
static const NSString *ofNoCommand = @"";
static const NSString *ofCommandSeparator = @"|";
static const NSString *ofNotificationsKey = @"OFAllowNotifications";
static const NSString *ofChallengeToUserFile = @"OFChallengeToUser.dat";
static const NSString *ofChallengeDataFile = @"OFChallengeData.dat";

// Mono stuff needed for Unity script callbacks.
extern "C"
{
	typedef void* MonoDomain;
	typedef void* MonoAssembly;
	typedef void* MonoImage;
	typedef void* MonoClass;
	typedef void* MonoObject;
	typedef void* MonoMethodDesc;
	typedef void* MonoMethod;
	typedef void* MonoString;
	typedef void* MonoEvent;
	typedef void* gpointer;
	typedef int gboolean;
	
	MonoDomain *mono_domain_get();
	MonoAssembly *mono_domain_assembly_open(MonoDomain *domain, const char *assemblyName);
	MonoImage *mono_assembly_get_image(MonoAssembly *assembly);
	MonoMethodDesc *mono_method_desc_new(const char *methodString, gboolean useNamespace);
	void mono_method_desc_free(MonoMethodDesc *desc);
	MonoMethod *mono_method_desc_search_in_image(MonoMethodDesc *methodDesc, MonoImage *image);
	MonoObject *mono_runtime_invoke(MonoMethod *method, void *obj, void **params, MonoObject **exc);
	MonoString *mono_string_new(MonoDomain *domain, const char *cString);
}

MonoMethod *mono_get_method_from_image(const char *name, MonoImage *image);

// Mono pointers that we want to keep track of.
static MonoDomain *monoDomain = NULL;
static MonoImage *monoImage = NULL;

// The OpenFeint Unity methods from script.
static MonoMethod *ofTriggerGeneralEvent;
static MonoMethod *ofTriggerDeferredResultEvent;
static MonoMethod *ofTriggerNotificationEvent;
static MonoMethod *ofTriggerUserLogInEvent;
static MonoMethod *ofTriggerStartedChallengeEvent;
static MonoMethod *ofTriggerFollowingResultCallback;
static MonoMethod *ofTriggerChallengeDefinitionResultCallback;
static MonoMethod *ofTriggerCloudBlobDownloadResultCallback;

// Unity functions that we'll need.
bool gDidResignActive = false;
void UnityPause(bool pause);
void UnitySetAudioSessionActive(bool active);
void UnityCleanup();

// Globals, because apparently we can't add members to categories (???).
static BOOL unityPaused = NO;
static NSString *cloudBlobDownloadLocation;


// OpenFeint Category interface.
@interface AppController (OpenFeint) <OpenFeintDelegate, OFChallengeDelegate, OFNotificationDelegate>

- (void)commonInit:(UIApplication *)application;

- (void)triggerGeneralEvent:(eGeneralEvent)eventIndex;
- (void)triggerDeferredResultEvent:(eDeferredResultEvent)eventIndex result:(BOOL)result;
- (void)triggerNotificationEvent:(OFNotificationData *)notificationData;
- (void)triggerUserLogInEvent:(eUserLogInEvent)eventIndex userId:(NSString *)userId;

@end


// OpenFeint Category implementation.
@implementation AppController (OpenFeint)

- (void)commonInit:(UIApplication *)application
{
	if([self respondsToSelector:@selector(startRendering:)])
	{
		// Unity 1.5
		[self startUnity:application];
	}
	else
	{
		NSLog(@"The OpenFeint Unity package is not compatible with Unity versions below 1.5.");
		NSLog(@"Please upgrade to Unity iPhone version 1.5 or above, or use an older version\nof the OpenFeint Unity support.");
		exit(0);
		return;
	}

	// Clear the command and result keys just in case any
	// values were left over from a previous run of the game.
	[[NSUserDefaults standardUserDefaults] removeObjectForKey:ofCommandKey];
	[[NSUserDefaults standardUserDefaults] removeObjectForKey:ofResultKey];
	
	// Initialize OpenFeint.
	OFDelegatesContainer* delegates =
	[OFDelegatesContainer containerWithOpenFeintDelegate:self
									andChallengeDelegate:self
								 andNotificationDelegate:self];
	
	NSDictionary* settings = [NSDictionary dictionaryWithObjectsAndKeys:
							  [NSNumber numberWithInt:UOF_INITIAL_DASHBOARD_ORIENTATION], OpenFeintSettingDashboardOrientation,
							  UOF_APP_SHORT_DISPLAY_NAME, OpenFeintSettingShortDisplayName,
							  [NSNumber numberWithInt:UOF_NOTIFICATION_POSITION], OpenFeintSettingNotificationPosition,
							  [NSNumber numberWithBool:UOF_ENABLE_PUSH_NOTIFICATIONS], OpenFeintSettingEnablePushNotifications,
							  [NSNumber numberWithBool:UOF_DISABLE_USER_GENERATED_CONTENT], OpenFeintSettingDisableUserGeneratedContent,
							  [NSNumber numberWithBool:UOF_PROMPT_TO_POST_ACHIEVEMENT_UNLOCK], OpenFeintSettingPromptToPostAchievementUnlock,
							  [NSNumber numberWithBool:UOF_ALWAYS_ASK_FOR_APPROVAL_IN_DEBUG], OpenFeintSettingAlwaysAskForApprovalInDebug,
							  nil];
	
	[OpenFeint initializeWithProductKey:UOF_PRODUCT_KEY
							  andSecret:UOF_SECRET_KEY
						 andDisplayName:UOF_APP_DISPLAY_NAME
							andSettings:settings
						   andDelegates:delegates];
	
	// Set the initial value of whether or not we should allow OpenFeint notifications.
	// This way, we can change the value through PlayerPrefs at any time.
	[[NSUserDefaults standardUserDefaults] setInteger:((UOF_ALLOW_NOTIFICATIONS) ? 1 : 0) forKey:ofNotificationsKey];
	
	// Give ourselves access to the OpenFeint Unity scripts via Mono.
	NSString *path = [[[NSBundle mainBundle] bundlePath]
					  stringByAppendingPathComponent:@"Data/Managed/Assembly-CSharp-firstpass.dll"];
	monoDomain = mono_domain_get();
	MonoAssembly *csharp = mono_domain_assembly_open(monoDomain, path.UTF8String);
	monoImage = mono_assembly_get_image(csharp);
	
	// Get the OpenFeint Unity script methods from Mono.
	ofTriggerGeneralEvent = OF_METHOD("OpenFeint:TriggerGeneralEvent(int)");
	ofTriggerDeferredResultEvent = OF_METHOD("OpenFeint:TriggerDeferredResultEvent(int,bool)");
	ofTriggerNotificationEvent = OF_METHOD("OpenFeint:TriggerNotificationEvent(string,int,int)");
	ofTriggerUserLogInEvent = OF_METHOD("OpenFeint:TriggerUserLogInEvent(int,string)");
	ofTriggerStartedChallengeEvent = OF_METHOD("OpenFeint:TriggerStartedChallengeEvent(string,long,string,bool)");
	ofTriggerFollowingResultCallback = OF_METHOD("OpenFeint:TriggerFollowingResultCallback(bool,bool)");
	ofTriggerChallengeDefinitionResultCallback = OF_METHOD("OpenFeint:TriggerChallengeDefinitionResultCallback(bool)");
	ofTriggerCloudBlobDownloadResultCallback = OF_METHOD("OpenFeint:TriggerDownloadCloudBlobResultCallback(bool)");
	
	// Set up a notification so that we know when NSUserDefaults (PlayerPrefs) are changed.
	[[NSUserDefaults standardUserDefaults] addObserver:self forKeyPath:@"OFCommand" options:0 context:nil];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// Get the current command.
	NSString *OFCommand = [[NSUserDefaults standardUserDefaults] objectForKey:ofCommandKey];
	
	if(!OFCommand
	   || [OFCommand isEqualToString:ofNoCommand])
	{
		// No commands sent.
		return;
	}
	
	// Clear the result key.
	[[NSUserDefaults standardUserDefaults] removeObjectForKey:ofResultKey];
	
	// Split the command and argument list.
	NSArray *args = [OFCommand componentsSeparatedByString:ofCommandSeparator];
	
	// Store the command name for quick usage.
	OFCommand = [args objectAtIndex:0];
	
	// LaunchDashboard //
	if(OF_CMD(@"LaunchDashboard"))
	{
		int pageId = [[args objectAtIndex:1] intValue];
		
		switch(pageId)
		{
			case ePageLeaderboards:		[OpenFeint launchDashboardWithListLeaderboardsPage]; break;
			case ePageAchievements:		[OpenFeint launchDashboardWithAchievementsPage]; break;
			case ePageChallenges:		[OpenFeint launchDashboardWithChallengesPage]; break;
			case ePageFindFriends:		[OpenFeint launchDashboardWithFindFriendsPage]; break;
			case ePageWhosPlaying:		[OpenFeint launchDashboardWithWhosPlayingPage]; break;
			case ePageGlobalChatRooms:	[OpenFeint launchDashboardWithListGlobalChatRoomsPage]; break;
			case ePageSwitchUser:		[OpenFeint launchDashboardWithSwitchUserPage]; break;
				
			case ePageMain:
			default:
				[OpenFeint launchDashboard];
				break;
		}
	}
	// LaunchDashboardWithLeaderboardID //
	else if(OF_CMD(@"LaunchDashboardWithLeaderboardID"))
	{
		[OpenFeint launchDashboardWithHighscorePage:[args objectAtIndex:1]];
	}
	// SubmitHighScore //
	else if(OF_CMD(@"SubmitHighScore"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onHighScoreSuccess));
		OFDelegate failure(self, @selector(onHighScoreFailure));
		
		// Send the high score.
		[OFHighScoreService setHighScore:[[args objectAtIndex:1] longLongValue]
						 withDisplayText:[args objectAtIndex:2]
						  forLeaderboard:[args objectAtIndex:3]
								silently:[[args objectAtIndex:4] boolValue]
							   onSuccess:success
							   onFailure:failure];
	}
	// UnlockAchievement //
	else if(OF_CMD(@"UnlockAchievement"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onAchievementSuccess));
		OFDelegate failure(self, @selector(onAchievementFailure));
		
		// Unlock the achievement.
		[OFAchievementService updateAchievement:[args objectAtIndex:1]
							 andPercentComplete:100
							andShowNotification:FALSE
									  onSuccess:success
									  onFailure:failure];
	}
	// UpdateAchievement //
	else if(OF_CMD(@"UpdateAchievement"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onAchievementSuccess));
		OFDelegate failure(self, @selector(onAchievementFailure));
		
		// Unlock the achievement.
		[OFAchievementService updateAchievement:[args objectAtIndex:1]
							 andPercentComplete:[[args objectAtIndex:2] doubleValue]
							andShowNotification:[[args objectAtIndex:3] boolValue]
									  onSuccess:success
									  onFailure:failure];
	}
	// SendSocialNotification //
	else if(OF_CMD(@"SendSocialNotification"))
	{
		// Send the social notification.
		if([[args objectAtIndex:2] isEqualToString:ofNoCommand])
		{
			[OFSocialNotificationService sendWithText:[args objectAtIndex:1]];
		}
		else
		{
			[OFSocialNotificationService sendWithText:[args objectAtIndex:1] imageNamed:[args objectAtIndex:2]];
		}
	}
	// GetChallengeDataToPlayerPrefs //
	else if(OF_CMD(@"GetChallengeDataToPlayerPrefs"))
	{
		// Load the challenge data.
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
		NSData *challengeData = [NSData dataWithContentsOfFile:[[paths objectAtIndex:0] stringByAppendingPathComponent:ofChallengeDataFile]];

		// Create an array of our PlayerPrefs keys.
		NSMutableArray *keys = [[NSMutableArray alloc] initWithCapacity:[args count] - 1];
		for(int i = 1; i < [args count]; i++)
		{
			[keys addObject:[args objectAtIndex:i]];
		}
		
		// Unarchive the challenge data.
		NSArray *playerPrefsData = [NSKeyedUnarchiver unarchiveObjectWithData:challengeData];
		
		// Warn if the two arrays aren't the same size. This is not
		// considered an error as developers will need backwards
		// compatibility if their challenge data changes formats in future
		// updates.
		if([keys count] != [playerPrefsData count])
		{
			NSLog([NSString stringWithFormat:@"OpenFeint warning: Number of PlayerPrefs keys passed to GetChallengeDataToPlayerPrefs (%d) does not equal number of keys downloaded (%d).", [keys count], [playerPrefsData count]]);
		}
		
		// Now send this data back via the PlayerPrefs keys.
		for(int i = 0; i < [keys count] && i < [playerPrefsData count]; i++)
		{
			[[NSUserDefaults standardUserDefaults] setObject:[playerPrefsData objectAtIndex:i] forKey:[keys objectAtIndex:i]];
		}
	}
	else if(OF_CMD(@"GetChallengeDataToFile"))
	{
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);

		// Load the challenge data.
		NSData *challengeData = [NSData dataWithContentsOfFile:[[paths objectAtIndex:0] stringByAppendingPathComponent:ofChallengeDataFile]];
		
		// Now write it to where we want it.
		[challengeData writeToFile:[[paths objectAtIndex:0] stringByAppendingPathComponent:[args objectAtIndex:1]]
						atomically:YES];
	}
	// DisplaySendChallengeModalWithPlayerPrefs //
	else if(OF_CMD(@"DisplaySendChallengeModalWithPlayerPrefs"))
	{
		// First, put all of the PlayerPrefs keys into an array.
		NSMutableArray *playerPrefsData = [[NSMutableArray alloc] initWithCapacity:[args count] - 3];
		for(int i = 3; i < [args count]; i++)
		{
			[playerPrefsData addObject:[[NSUserDefaults standardUserDefaults] objectForKey:[args objectAtIndex:i]]];
		}
		
		// Create the challenge data out of the array.
		NSData *challengeData = [NSKeyedArchiver archivedDataWithRootObject:playerPrefsData];
		
		// Now display the modal with the challenge data.
		[OFChallengeService displaySendChallengeModal:[args objectAtIndex:1]
										challengeText:[args objectAtIndex:2]
										challengeData:challengeData];
		
		// Clean up.
		[playerPrefsData release];
	}
	// DisplaySendChallengeModalWithFile //
	else if(OF_CMD(@"DisplaySendChallengeModalWithFile"))
	{
		// Get the path to the file. The file will be stored in the Documents directory.
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
		NSString *filePath = [[paths objectAtIndex:0] stringByAppendingPathComponent:[args objectAtIndex:2]];
		
		// Create the challenge data from the file.
		NSData *challengeData = [NSData dataWithContentsOfFile:filePath];
		
		// Now display the modal with the challenge data.
		[OFChallengeService displaySendChallengeModal:[args objectAtIndex:1]
										challengeText:[args objectAtIndex:2]
										challengeData:challengeData];
	}
	// SubmitChallengeResult //
	else if(OF_CMD(@"SubmitChallengeResult"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onChallengeResultSuccess));
		OFDelegate failure(self, @selector(onChallengeResultFailure));
		
		// Get the challenge to user from the serialized file.
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
		OFChallengeToUser *challengeToUser = [OFChallengeService readChallengeToUserFromFile:[[paths objectAtIndex:0] stringByAppendingPathComponent:ofChallengeToUserFile]];
		
		// Submit the challenge result.
		[OFChallengeService submitChallengeResult:[challengeToUser resourceId]
										   result:(OFChallengeResult)[[args objectAtIndex:1] intValue]
								resultDescription:[args objectAtIndex:2]
										onSuccess:success
										onFailure:failure];
	}
	// DisplayChallengeCompletedModalWithPlayerPrefs //
	else if(OF_CMD(@"DisplayChallengeCompletedModalWithPlayerPrefs"))
	{
		// Get the challenge to user from the serialized file.
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
		OFChallengeToUser *challengeToUser = [OFChallengeService readChallengeToUserFromFile:[[paths objectAtIndex:0] stringByAppendingPathComponent:ofChallengeToUserFile]];
		
		NSData *challengeData = nil;
		NSMutableArray *playerPrefsData = nil;
		if([args count] > 3)
		{
			// Put all of the PlayerPrefs keys into an array.
			NSMutableArray *playerPrefsData = [[NSMutableArray alloc] initWithCapacity:[args count] - 4];
			for(int i = 4; i < [args count]; i++)
			{
				[playerPrefsData addObject:[[NSUserDefaults standardUserDefaults] objectForKey:[args objectAtIndex:i]]];
			}
			
			challengeData = [NSKeyedArchiver archivedDataWithRootObject:playerPrefsData];
		}
		challengeToUser.result = (OFChallengeResult)[[args objectAtIndex:1] intValue];


		// Now display the modal.
		[OFChallengeService displayChallengeCompletedModal:challengeToUser
												resultData:challengeData
										 resultDescription:[args objectAtIndex:2] 
									reChallengeDescription:[args objectAtIndex:3]];
		
		// Clean up.
		if(playerPrefsData != nil)
		{
			[playerPrefsData release];
		}
	}
	// DisplaySendChallengeCompletedModalWithFile //
	else if(OF_CMD(@"DisplayChallengeCompletedModalWithFile"))
	{
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);

		// Get the challenge to user for the serialized file.
		OFChallengeToUser *challengeToUser = [OFChallengeService readChallengeToUserFromFile:[[paths objectAtIndex:0] stringByAppendingPathComponent:ofChallengeToUserFile]];
		
		// Get the path to the file. The file will be stored in the Documents directory.
		NSString *filePath = [[paths objectAtIndex:0] stringByAppendingPathComponent:[args objectAtIndex:4]];
		
		// Create the challenge blob from the file.
		NSData *challengeData = [NSData dataWithContentsOfFile:filePath];
		challengeToUser.result = (OFChallengeResult)[[args objectAtIndex:1] intValue];
		// Now display the modal.
		[OFChallengeService displayChallengeCompletedModal:challengeToUser
												resultData:challengeData
										 resultDescription:[args objectAtIndex:2] 
									reChallengeDescription:[args objectAtIndex:3]];
	}
	// GetChallengeDefinition //
	else if(OF_CMD(@"GetChallengeDefinition"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onChallengeDefinitionSuccess:));
		OFDelegate failure(self, @selector(onChallengeDefinitionFailure));
		
		// Get the challenge definition.
		[OFChallengeDefinitionService getChallengeDefinitionWithId:[args objectAtIndex:1]
														 onSuccess:success
														 onFailure:failure];
	}
	// UploadBlobFromPlayerPrefsKeys //
	else if(OF_CMD(@"UploadBlobFromPlayerPrefs"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onUploadBlobSuccess));
		OFDelegate failure(self, @selector(onUploadBlobFailure));
		
		// First, put all of the PlayerPrefs keys into an array.
		NSMutableArray *playerPrefsData = [[NSMutableArray alloc] initWithCapacity:[args count] - 2];
		for(int i = 2; i < [args count]; i++)
		{
			[playerPrefsData addObject:[[NSUserDefaults standardUserDefaults] objectForKey:[args objectAtIndex:i]]];
		}
		
		// Create a data blob out of the array.
		NSData *dataBlob = [NSKeyedArchiver archivedDataWithRootObject:playerPrefsData];
		
		// Now upload the blob to the cloud.
		[OFCloudStorageService uploadBlob:dataBlob
								  withKey:[args objectAtIndex:1]
								onSuccess:success
								onFailure:failure];
		
		// Clean up.
		[playerPrefsData release];
	}
	// UploadBlobFromFilePath //
	else if(OF_CMD(@"UploadBlobFromFile"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onUploadBlobSuccess));
		OFDelegate failure(self, @selector(onUploadBlobFailure));

		// Get the path to the file. The file will be stored in the Documents directory.
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
		NSString *filePath = [[paths objectAtIndex:0] stringByAppendingPathComponent:[args objectAtIndex:2]];
		
		// Create the data blob from the file.
		NSData *dataBlob = [NSData dataWithContentsOfFile:filePath];
		
		// Now upload the blob to the cloud.
		[OFCloudStorageService uploadBlob:dataBlob
								  withKey:[args objectAtIndex:1]
								onSuccess:success
								onFailure:failure];
	}
	// DownloadBlobToPlayerPrefsKeys //
	else if(OF_CMD(@"DownloadBlobToPlayerPrefs"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onDownloadBlobToPlayerPrefsSuccess:));
		OFDelegate failure(self, @selector(onDownloadBlobFailure));
		
		// Set up the cloud blob download location.
		NSMutableString *aString = [[NSMutableString alloc] initWithString:@""];
		for(int i = 2; i < [args count]; i++)
		{
			if(i != 2)
			{
				[aString appendString:@"|"];
			}
			
			[aString appendString:[args objectAtIndex:i]];
		}
		
		cloudBlobDownloadLocation = aString;
		
		// Download the blob from the cloud.
		[OFCloudStorageService downloadBlobWithKey:[args objectAtIndex:1]
										 onSuccess:success
										 onFailure:failure];
	}
	// DownloadBlobToFilePath //
	else if(OF_CMD(@"DownloadBlobToFile"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onDownloadBlobToFilePathSuccess:));
		OFDelegate failure(self, @selector(onDownloadBlobFailure));
		
		// Set up the cloud blob download location.
		cloudBlobDownloadLocation = [[NSString alloc] initWithString:[args objectAtIndex:2]];
		
		// Download the blob from the cloud.
		[OFCloudStorageService downloadBlobWithKey:[args objectAtIndex:1]
										 onSuccess:success
										 onFailure:failure];
	}
	// SetDashboardOrientation //
	else if(OF_CMD(@"SetDashboardOrientation"))
	{
		// Set the dashboard orientation.
		[OpenFeint setDashboardOrientation:(UIInterfaceOrientation)[[args objectAtIndex:1] intValue]];
	}
	// PresentUserFeintApprovalModal //
	else if(OF_CMD(@"PresentUserFeintApprovalModal"))
	{
		// Set up the delegates.
		OFDelegate approval(self, @selector(onDoNothing:));
		OFDelegate denial(self, @selector(onDoNothing));
		
		// Present the OpenFeint approval screen.
		[OpenFeint presentUserFeintApprovalModal:approval deniedDelegate:denial];
	}
	// IsLocalUserFollowingAnyone //
	else if(OF_CMD(@"IsLocalUserFollowingAnyone"))
	{
		// Set up the delegates.
		OFDelegate success(self, @selector(onLocalUserFollowingSuccess:));
		OFDelegate failure(self, @selector(onLocalUserFollowingFailure));
		
		// Start the check.
		[OFFriendsService isLocalUserFollowingAnyone:success onFailure:failure];
	}
	// HasUserApprovedFeint //
	else if(OF_CMD(@"HasUserApprovedFeint"))
	{
		[[NSUserDefaults standardUserDefaults] setInteger:([OpenFeint hasUserApprovedFeint] ? 1 : 0)
												   forKey:ofResultKey];
	}
	// IsOnline //
	else if(OF_CMD(@"IsOnline"))
	{
		[[NSUserDefaults standardUserDefaults] setInteger:([OpenFeint isOnline] ? 1 : 0)
												   forKey:ofResultKey];
	}
	// LocalUserID //
	else if(OF_CMD(@"LocalUserID"))
	{
		[[NSUserDefaults standardUserDefaults] setObject:[OpenFeint lastLoggedInUserId] forKey:ofResultKey];
	}
	// LocalUserName //
	else if(OF_CMD(@"LocalUserName"))
	{
		[[NSUserDefaults standardUserDefaults] setObject:[OpenFeint lastLoggedInUserName] forKey:ofResultKey];
	}
	// UserAllowsNotifications //
	else if(OF_CMD(@"UserAllowsNotifications"))
	{
		[[NSUserDefaults standardUserDefaults] setInteger:([OpenFeint userAllowsNotifications] ? 1 : 0)
												   forKey:ofResultKey];
	}
	else
	{
		NSLog([NSString stringWithFormat:@"OpenFeint error: Invalid Unity command - %@", OFCommand]);
	}
}

- (void)triggerGeneralEvent:(eGeneralEvent)eventIndex
{
	void *args[1] = { &eventIndex };
	mono_runtime_invoke(ofTriggerGeneralEvent, NULL, args, NULL);
}

- (void)triggerDeferredResultEvent:(eDeferredResultEvent)eventIndex result:(BOOL)result
{
	void *args[2] = { &eventIndex, &result };
	mono_runtime_invoke(ofTriggerDeferredResultEvent, NULL, args, NULL);
}

- (void)triggerUserLogInEvent:(eUserLogInEvent)eventIndex userId:(NSString *)userId
{
	const char *text = userId.UTF8String;
	void *args[2] = { &eventIndex, mono_string_new(monoDomain, text) };
	
	mono_runtime_invoke(ofTriggerUserLogInEvent, NULL, args, NULL);
}

- (void)triggerNotificationEvent:(OFNotificationData *)notificationData
{
	const char *text = notificationData.notificationText.UTF8String;
	int category = notificationData.notificationCategory;
	int type = notificationData.notificationType;
	void *args[3] = { mono_string_new(monoDomain, text), &category, &type };
	
	mono_runtime_invoke(ofTriggerNotificationEvent, NULL, args, NULL);
}


// UIApplicationDelegate //
- (void)applicationDidFinishLaunching:(UIApplication *)application
{
    [self commonInit:application];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    [self commonInit:application];
    [OpenFeint respondToApplicationLaunchOptions:launchOptions];
    return YES;
}

- (void)applicationWillTerminate:(UIApplication *)application
{
	printf_console("-> applicationWillTerminate()\n");

	// Clean up Unity.
	UnityCleanup();

	// Stop receiving notifications.
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	// Ensure that OpenFeint is shutdown.
	[OpenFeint shutdown];
}																
																
- (void) applicationDidBecomeActive:(UIApplication*)application
{
	printf_console("-> applicationDidBecomeActive()\n");

    if (gDidResignActive == true)
    {
	    UnitySetAudioSessionActive(true);
	    UnityPause(false);
    }
	
    gDidResignActive = false;
	
	[OpenFeint applicationDidBecomeActive];
}

- (void) applicationWillResignActive:(UIApplication*)application
{
	[OpenFeint applicationWillResignActive];

	printf_console("-> applicationDidResignActive()\n");
	UnitySetAudioSessionActive(false);
	UnityPause(true);
    gDidResignActive = true;
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    [OpenFeint applicationDidRegisterForRemoteNotificationsWithDeviceToken:deviceToken];
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    OFLog([NSString stringWithFormat:@"Failed to register for remote notifications with error: %@", [error localizedDescription]]);
    [OpenFeint applicationDidFailToRegisterForRemoteNotifications];
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo
{
    [OpenFeint applicationDidReceiveRemoteNotification:userInfo];
}


// OpenFeintDelegate //
- (void)dashboardWillAppear
{
	unityPaused = YES;
	UnitySetAudioSessionActive(false);
	UnityPause(true);
}

- (void)dashboardDidDisappear
{
	unityPaused = NO;
	UnitySetAudioSessionActive(true);
	UnityPause(false);
}

- (void)userLoggedIn:(NSString *)userId
{
	[self triggerUserLogInEvent:eUserLogInEventOnUserLoggedIn userId:userId];
}

- (void)userLoggedOut:(NSString *)userId
{
	[self triggerUserLogInEvent:eUserLogInEventOnUserLoggedOut userId:userId];
}

- (BOOL)canReceiveCallbacksNow
{
	return YES;
}


// OFNotificationDelegate //
- (BOOL)isOpenFeintNotificationAllowed:(OFNotificationData*)notificationData
{
	return [[NSUserDefaults standardUserDefaults] integerForKey:ofNotificationsKey];
}

- (void)handleDisallowedNotification:(OFNotificationData*)notificationData
{
	[self triggerNotificationEvent:notificationData];
}

- (void)notificationWillShow:(OFNotificationData*)notificationData
{
	[self triggerNotificationEvent:notificationData];
}


// OFChallengeDelegate //
- (void)userLaunchedChallenge:(OFChallengeToUser*)challengeToLaunch
			withChallengeData:(NSData*)challengeData
{
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);

	// Write the challenge to user data to a file.
	[OFChallengeService writeChallengeToUserToFile:[[paths objectAtIndex:0] stringByAppendingPathComponent:ofChallengeToUserFile]
								   challengeToUser:challengeToLaunch];
	
	// Write the challenge data to a file so that it's automatically
	// serialized, and we can get to it at any time while the challenge
	// is still active.
	[challengeData writeToFile:[[paths objectAtIndex:0] stringByAppendingPathComponent:ofChallengeDataFile]
					atomically:YES];
	
	// Trigger the started challenge event, and send along the challenge
	// definition while we're at it.
	OFChallengeDefinition *challengeDefinition = [[challengeToLaunch challenge] challengeDefinition];
	const char *title = challengeDefinition.title.UTF8String;
	int64_t challengeId = [[challengeDefinition resourceId] longLongValue];
	const char *description = challengeDefinition.description.UTF8String;
	BOOL multiAttempt = challengeDefinition.multiAttempt;
	void *args[4] = { mono_string_new(monoDomain, title), &challengeId, mono_string_new(monoDomain, description), &multiAttempt };
	
	mono_runtime_invoke(ofTriggerStartedChallengeEvent, NULL, args, NULL);
}

- (void)userRestartedChallenge
{
	[self triggerGeneralEvent:eGeneralEventOnRestartedChallenge];
}

#if UOF_ALLOW_STRONGER_ATTEMPT_FOR_CHALLENGES
- (void)userRestartedCreateChallenge
{
	[self triggerGeneralEvent:eGeneralEventOnRestartedCreateChallenge];
}
#endif // UOF_ALLOW_STRONGER_ATTEMPT_FOR_CHALLENGES

- (void)completedChallengeScreenClosed
{
	[self triggerGeneralEvent:eGeneralEventOnCompletedChallengeScreenClosed];
}

- (void)sendChallengeScreenClosed
{
	[self triggerGeneralEvent:eGeneralEventOnSendChallengeScreenClosed];
}

- (void)userBootedWithUnviewedChallenges:(NSUInteger)numChallenges
{
	[[NSUserDefaults standardUserDefaults] setInteger:numChallenges forKey:@"OFBootedWithUnviewedChallenges"];
}

- (void)userSentChallenges
{
	[self triggerGeneralEvent:eGeneralEventOnUserSentChallenges];
}


// Success/failure OFDelegates //
- (void)onChallengeDefinitionSuccess:(OFPaginatedSeries *)series
{
	OFChallengeDefinition *definition = [series objectAtIndex:0];
	
	// Sending this back through PlayerPrefs instead of a struct
	// because we don't know how memory is managed in Mono, and
	// Unity iPhone Basic users don't have access to InteropServices.
	[[NSUserDefaults standardUserDefaults] setObject:definition.title forKey:@"OFChallengeDefinition_title"];
	[[NSUserDefaults standardUserDefaults] setInteger:[[definition resourceId] longLongValue] forKey:@"OFChallengeDefinition_challengeId"];
	[[NSUserDefaults standardUserDefaults] setObject:definition.iconUrl forKey:@"OFChallengeDefinition_iconUrl"];
	[[NSUserDefaults standardUserDefaults] setInteger:definition.multiAttempt forKey:@"OFChallengeDefinition_multiAttempt"];
	
	BOOL result = YES;
	void *args[1] = { &result };
	
	mono_runtime_invoke(ofTriggerChallengeDefinitionResultCallback, NULL, args, NULL);
}

- (void)onChallengeDefinitionFailure
{
	BOOL result = NO;
	void *args[1] = { &result };
	
	mono_runtime_invoke(ofTriggerChallengeDefinitionResultCallback, NULL, args, NULL);
}

- (void)onLocalUserFollowingSuccess:(NSNumber *)commandResult
{
	BOOL success = YES;
	BOOL result = [commandResult boolValue];
	void *args[2] = { &success, &result };
	
	mono_runtime_invoke(ofTriggerFollowingResultCallback, NULL, args, NULL);
}

- (void)onLocalUserFollowingFailure
{
	BOOL success = NO;
	BOOL result = NO;
	void *args[2] = { &success, &result };
	
	mono_runtime_invoke(ofTriggerFollowingResultCallback, NULL, args, NULL);
}

- (void)onDownloadBlobToPlayerPrefsSuccess:(NSData *)dataBlob
{
	// Split the location string into a series of PlayerPrefs keys.
	NSArray *keys = [cloudBlobDownloadLocation componentsSeparatedByString:ofCommandSeparator];

	// Unarchive the data blob.
	NSArray *playerPrefsData = [NSKeyedUnarchiver unarchiveObjectWithData:dataBlob];
	
	// Warn if the two arrays aren't the same size. This is not
	// considered an error as developers will need backwards
	// compatibility if their blob data changes formats in future
	// updates.
	if([keys count] != [playerPrefsData count])
	{
		NSLog([NSString stringWithFormat:@"OpenFeint warning: Number of PlayerPrefs keys passed to DownloadBlobToPlayerPrefsKeys (%d) does not equal number of keys downloaded (%d).", [keys count], [playerPrefsData count]]);
	}
	
	// Now send this data back via the PlayerPrefs keys.
	for(int i = 0; i < [keys count] && i < [playerPrefsData count]; i++)
	{
		[[NSUserDefaults standardUserDefaults] setObject:[playerPrefsData objectAtIndex:i] forKey:[keys objectAtIndex:i]];
	}

	// Clean up and trigger our callback.
	BOOL success = YES;
	void *args[1] = { &success };
	mono_runtime_invoke(ofTriggerCloudBlobDownloadResultCallback, NULL, args, NULL);
	[cloudBlobDownloadLocation release];
}

- (void)onDownloadBlobToFilePathSuccess:(NSData *)dataBlob
{
	// Get the path to the file. The file will be stored in the Documents directory.
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *filePath = [[paths objectAtIndex:0] stringByAppendingPathComponent:cloudBlobDownloadLocation];
	
	// Write the data blob to a file.
	[dataBlob writeToFile:filePath atomically:YES];
	
	// Clean up and trigger our callback.
	BOOL success = YES;
	void *args[1] = { &success };
	mono_runtime_invoke(ofTriggerCloudBlobDownloadResultCallback, NULL, args, NULL);
	[cloudBlobDownloadLocation release];
}

- (void)onDownloadBlobFailure
{
	// Clean up and trigger our callback.
	BOOL success = NO;
	void *args[1] = { &success };
	mono_runtime_invoke(ofTriggerCloudBlobDownloadResultCallback, NULL, args, NULL);
	[cloudBlobDownloadLocation release];
}

- (void)onHighScoreSuccess			{ [self triggerDeferredResultEvent:eDeferredResultEventOnHighScoreSubmitted result:YES]; }
- (void)onHighScoreFailure			{ [self triggerDeferredResultEvent:eDeferredResultEventOnHighScoreSubmitted result:NO]; }
- (void)onAchievementSuccess		{ [self triggerDeferredResultEvent:eDeferredResultEventOnAchievementUnlocked result:YES]; }
- (void)onAchievementFailure		{ [self triggerDeferredResultEvent:eDeferredResultEventOnAchievementUnlocked result:NO]; }
- (void)onChallengeResultSuccess	{ [self triggerDeferredResultEvent:eDeferredResultEventOnChallengeResultSubmitted result:YES]; }
- (void)onChallengeResultFailure	{ [self triggerDeferredResultEvent:eDeferredResultEventOnChallengeResultSubmitted result:NO]; }
- (void)onUploadBlobSuccess			{ [self triggerDeferredResultEvent:eDeferredResultEventOnUploadedCloudBlob result:YES]; }
- (void)onUploadBlobFailure			{ [self triggerDeferredResultEvent:eDeferredResultEventOnUploadedCloudBlob result:NO]; }
- (void)onDoNothing { }
- (void)onDoNothing:(BOOL)nothing { }

// Mono utility functions //
MonoMethod *mono_get_method_from_image(const char *name, MonoImage *image)
{
	MonoMethodDesc *desc = mono_method_desc_new(name, FALSE);
	MonoMethod *method = mono_method_desc_search_in_image(desc, monoImage);
	mono_method_desc_free(desc);
	
	return method;
}

@end

