////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// 
///  Copyright 2010 Aurora Feint, Inc.
/// 
///  Licensed under the Apache License, Version 2.0 (the "License");
///  you may not use this file except in compliance with the License.
///  You may obtain a copy of the License at
///  
///      http://www.apache.org/licenses/LICENSE-2.0
///      
///  Unless required by applicable law or agreed to in writing, software
///  distributed under the License is distributed on an "AS IS" BASIS,
///  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
///  See the License for the specific language governing permissions and
///  limitations under the License.
/// 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
// OpenFeint.cs
//
// Allows script access to the OpenFeint API. The methods, events, and delegates
// in this file can all be accessed from either C# or JavaScript.
//
// This code was contributed to OpenFeint by Tiny Tim Games (www.tinytimgames.com).
// It is thanks to awesome developers like these that OpenFeint can keep rocking!
//

using UnityEngine;
using System.Collections;
using System;

public class OpenFeint
{
	// ENUMERATIONS //
	
	public enum eDashboardPage
	{
		Main,
		Leaderboards,
		Achievements,
		Challenges,
		FindFriends,
		WhosPlaying,
		GlobalChatRooms,
		GameChatRooms,
		SwitchUser,
	}
	
	// The following enumerations correspond to enumerations found in the OpenFeint
	// source code.
	public enum eNotificationCategory	// Corresponds to OFNotificationCategory in OFNotificationDelegate.h
	{
		Foreground = 1,
		Login,
		Challenge,
		HighScore,
		Leaderboard,
		Achievement,
		SocialNotification,
		Presence,
	}
	
	public enum eNotificationType		// Corresponds to OFNotificationType in OFNotificationDelegate.h
	{
		None = 0,
		Submitting,
		Downloading,
		Error,
		Success,
		NewResources,
		UserPresenceOnline,
		UserPresenceOffline,
		NewMessage,
	}
	
	public enum eChallengeResult		// Corresponds to OFChallengeResult in OFChallengeToUser.h
	{
		Incomplete,
		RecipientWon,
		RecipientLost,
		Tie,
	}
	
	
	// STRUCTURES //
	
	public struct ChallengeDefinition
	{
		public string title;
		public long challengeId;
		public string iconUrl;
		public bool multiAttempt;
	}
	

	// DELEGATE DEFINITIONS //
	
	// Event handler delegates.
	public delegate void GeneralEventHandler();
	public delegate void DeferredResultEventHandler(bool result);
	public delegate void NotificationEventHandler(string text, eNotificationCategory category, eNotificationType type);
	public delegate void UserLogInEventHandler(string userId);
	public delegate void StartedChallengeEventHandler(ChallengeDefinition definition);
	
	// Callback delegates.
	public delegate void FollowingResultDelegate(bool success, bool isFollowingAnyone);
	public delegate void ChallengeDefinitionResultDelegate(bool success, ChallengeDefinition definition);
	public delegate void DeferredResultDelegate(bool success);
	

	// EVENTS //
	// Subscribe to these events to act upon particular actions from OpenFeint.
	//
	// NOTE: You MUST handle your own unsubscription from these events. If you
	// do not, an invalid reference exception will be thrown if the object
	// that contains your method is destroyed (either by you or during a level
	// load).
	//
	// Unsubscribe from the event as soon as you no longer need it (OnDisable
	// is a good place to put this). If you need to subscribe to an event across
	// level loads, either use a static method or use DontDestroyOnLoad on the
	// object.
	//
	// C# Example:
	//		// Method signature must match that of the NotificationEventHandler delegate.
	//		private void MyNotificationHandler(string text, OpenFeint.eNotificationCategory category, OpenFeint.eNotificationType type)
	//		{
	//			Debug.Log("Notification: " + text);
	//		}
	//
	//		// When you want to subscribe to the event (perhaps in Awake or Start).
	//		OpenFeint.OnNotification += new OpenFeint.NotificationEventHandler(MyNotificationHandler);
	//
	//		// When you want to unsubscribe from the event.
	//		OpenFeint.OnNotification -= MyNotificationHandler;
	//
	//
	// Javascript Example:
	//		// Function signature must match that of the NotificationEventHandler delegate.
	//		function MyNotificationHandler(text : string, category : OpenFeint.eNotificationCategory, type : OpenFeint.eNotificationType)
	//		{
	//			Debug.Log("Notification: " + text);
	//		}
	//
	//		// When you want to subscribe to the event (perhaps in Awake or Start).
	//		OpenFeint.OnNotification += OpenFeint.NotificationEventHandler(MyNotificationHandler);
	//
	//		// When you want to unsubscribe from the event.
	//		OpenFeint.OnNotification -= MyNotificationHandler;
	//
	
	// Sent after the user goes through the OpenFeint approval screen.
	static public event GeneralEventHandler OnFeintApprovalScreenClosed;
	
	// Sent when an OpenFeint notification should appear on screen. This is
	// sent regardless of whether or not notifications are allowed. If
	// notifications are not allowed, use this event to determine when you
	// should display your own message.
	static public event NotificationEventHandler OnNotification;
	
	// Sent upon success or failure of a high score submission.
	static public event DeferredResultEventHandler OnHighScoreSubmitted;
	
	// Sent upon success or failure of an achievement unlock. If the
	// achievement has already been unlocked locally, this event will not
	// be sent.
	static public event DeferredResultEventHandler OnAchievementUnlocked;
	
	// Sent when the user starts a challenge. You will need to subsequently
	// call one of the GetChallengeData method to get the challenge data that
	// was sent with the challenge.
	static public event StartedChallengeEventHandler OnStartedChallenge;
	
	// Sent when the user restarts a challenge.
	static public event GeneralEventHandler OnRestartedChallenge;
	
	// Sent when the user restarts a challenge that he is trying to create.
	// This message is only sent if stronger attempts are enabled for
	// challenges and if the challenge was a multi-attempt challenge.
	static public event GeneralEventHandler OnRestartedCreateChallenge;
	
	// Sent when the completed challenge screen is closed.
	static public event GeneralEventHandler OnCompletedChallengeScreenClosed;
	
	// Sent when the send challenge screen is closed.
	static public event GeneralEventHandler OnSendChallengeScreenClosed;
	
	// Sent when the user sends a challenge to another user.
	static public event GeneralEventHandler OnUserSentChallenges;
	
	// Sent upon success or failure of a challenge result submission.
	static public event DeferredResultEventHandler OnChallengeResultSubmitted;
	
	// Send upon success or failure of uploading a blob to the cloud.
	static public event DeferredResultEventHandler OnUploadedCloudBlob;
	
	// Sent when a user successfully connects to the OpenFeint servers.
	static public event UserLogInEventHandler OnUserLoggedIn;
	
	// Sent when a user explicitly logs out from OpenFeint (though does not
	// get sent when user switches accounts; only OnUserLoggedIn gets sent
	// in that instance).
	static public event UserLogInEventHandler OnUserLoggedOut;
	
	
	// METHODS //
	// These methods can be called directly from your scripts.
	//
	// Example:
	//		OpenFeint.LaunchDashboard(OpenFeint.eDashboardPage.Leaderboards);
	//
	
	// LaunchDashboard //
	//
	// Launches the OpenFeint Dashboard.
	// 	- dashboardPage:	The page to open the dashboard to.
	//
	static public void LaunchDashboard(eDashboardPage dashboardPage)
	{
		PerformCommand(String.Format("LaunchDashboard|{0}", (int)dashboardPage));
	}
	
	static public void LaunchDashboard()
	{
		LaunchDashboard(eDashboardPage.Main);
	}
	
	// LaunchDashboardWithLeaderboardID //
	//
	// Launch the OpenFeint Dashboard to a specific Leaderboard page.
	//	- leaderboardID:	The desired leaderboard ID.
	//
	static public void LaunchDashboardWithLeaderboardID(string leaderboardId)
	{
		PerformCommand(String.Format("LaunchDashboardWithLeaderboardID|{0}", leaderboardId));
	}
	
	static public void LaunchDashboardWithLeaderboardID(long leaderboardId)
	{
		LaunchDashboardWithLeaderboardID(leaderboardId.ToString());
	}
	
	// SubmitHighScore //
	//
	// Submit a high score to a specific leaderboard.
	//	- score:			The integer score.
	//	- displayText:		The text to display for this score instead of the integer score value.
	//	- leaderboardId:	String containing the leaderboard ID where the score is to be submitted.
	//	- silently:			Whether or not to display a notification upon successful high score submission.
	//
	//	events				OnHighScoreSubmitted
	//
	static public void SubmitHighScore(long score, string displayText, string leaderboardId, bool silently)
	{
		PerformCommand(String.Format("SubmitHighScore|{0}|{1}|{2}|{3}", score, displayText, leaderboardId, silently));
	}
	
	static public void SubmitHighScore(long score, string displayText, long leaderboardId, bool silently)
	{
		SubmitHighScore(score, displayText, leaderboardId.ToString(), silently);
	}
	
	static public void SubmitHighScore(long score, string displayText, string leaderboardId)
	{
		SubmitHighScore(score, displayText, leaderboardId, false);
	}
	
	static public void SubmitHighScore(long score, string displayText, long leaderboardId)
	{
		SubmitHighScore(score, displayText, leaderboardId.ToString(), false);
	}
	
	static public void SubmitHighScore(long score, string leaderboardId)
	{
		SubmitHighScore(score, score.ToString(), leaderboardId, false);
	}

	static public void SubmitHighScore(long score, long leaderboardId)
	{
		SubmitHighScore(score, score.ToString(), leaderboardId.ToString(), false);
	}
	
	// UnlockAchievement //
	//
	// Unlock an achievement.
	//	- achievementId:	The ID of the achievement.
	//
	//	events				OnAchievementUnlocked
	//
	static public void UnlockAchievement(string achievementId)
	{
		PerformCommand(String.Format("UnlockAchievement|{0}", achievementId));
	}

	static public void UnlockAchievement(long achievementId)
	{
		UnlockAchievement(achievementId.ToString());
	}
	
	// SendSocialNotification //
	//
	// Send a social notification to Twitter and Facebook.
	//	- text:					The text to send.
	//	- imageIdentifierName:	The identifier name of the image to send with the text.
	//
	static public void SendSocialNotification(string text, string imageIdentifierName)
	{
		PerformCommand(String.Format("SendSocialNotification|{0}|{1}", text, imageIdentifierName));
	}
	
	static public void SendSocialNotification(string text)
	{
		SendSocialNotification(text, "");
	}
	
	// GetChallengeDataToPlayerPrefs //
	//
	// Get the challenge data that was sent with a challenge.
	//	- playerPrefsKeys:		An array of PlayerPrefs keys where the challenge data for this
	//							challenge will be stored to.
	//
	static public void GetChallengeDataToPlayerPrefs(string[] playerPrefsKeys)
	{
		string commandString = "GetChallengeDataToPlayerPrefs";
		
		for(int i = 0; i < playerPrefsKeys.Length; i++)
		{
			commandString += "|" + playerPrefsKeys[i];
		}
		
		PerformCommand(commandString);
	}
	
	// GetChallengeDataToFile //
	//
	// Get the challenge data that was sent with a challenge.
	//	- filePath:				The path to a file in the app's Documents directory that will hold
	//							the challenge data.
	//
	static public void GetChallengeDataToFile(string filePath)
	{
		PerformCommand(String.Format("GetChallengeDataToFile|{0}", filePath));
	}

	// DisplaySendChallengeModalWithPlayerPrefs //
	//
	// Displays the modal for sending a challenge to another player.
	//	- challengeId:			The ID of the challenge.
	//	- challengeText:		Displays what needs to be fulfilled to complete the challenge.
	//	- dataKeys:				An array of PlayerPrefs keys where the data for this challenge
	//							is stored. This will be packaged up an sent as the challenge
	//							data from this player.
	//	- dataFilePath:			The path to a file located in the app's Documents directory
	//							that contains the challenge data. This is the preferred method
	//							for serialized XML data (Unity iPhone Advanced only).
	//
	static public void DisplaySendChallengeModalWithPlayerPrefs(string challengeId, string challengeText, string[] dataKeys)
	{
		string commandString = String.Format("DisplaySendChallengeModalWithPlayerPrefs|{0}|{1}", challengeId, challengeText);
		
		for(int i = 0; i < dataKeys.Length; i++)
		{
			commandString += "|" + dataKeys[i];
		}
		
		PerformCommand(commandString);
	}

	static public void DisplaySendChallengeModalWithPlayerPrefs(long challengeId, string challengeText, string[] dataKeys)
	{
		DisplaySendChallengeModalWithPlayerPrefs(challengeId.ToString(), challengeText, dataKeys);
	}
	
	// DisplaySendChallengeModalWithFile //
	//
	// Displays the modal for sending a challenge to another player.
	//	- challengeId:			The ID of the challenge.
	//	- challengeText:		Displays what needs to be fulfilled to complete the challenge.
	//	- dataFilePath:			The path to a file located in the app's Documents directory
	//							that contains the challenge data. This is the preferred method
	//							for serialized XML data (Unity iPhone Advanced only).
	//
	static public void DisplaySendChallengeModalWithFile(string challengeId, string challengeText, string dataFilePath)
	{
		PerformCommand(String.Format("DisplaySendChallengeModalWithFile|{0}|{1}|{2}", challengeId, challengeText, dataFilePath));
	}

	static public void DisplaySendChallengeModalWithFile(long challengeId, string challengeText, string dataFilePath)
	{
		DisplaySendChallengeModalWithFile(challengeId.ToString(), challengeText, dataFilePath);
	}
	
	// SubmitChallengeResult //
	//
	// Submits the result of a played challenge to the server. This should always be called
	// after the challenge game play session is over, regardless of the result or the type
	// of challenge. It should be called before DisplayChallengeCompletedModal.
	//	- result:				The result of the challenge.
	//	- resultDescription:	A string that will be prefixed by either the recipient's name
	//							or "You", depending on the player that is viewing the result.
	//							It should not contain whether the recipient won or lost, but
	//							should contain the statistics of his attempt. (Ex. "beat 30
	//							monsters" will turn into "You beat 30 monsters")
	//
	//	events					OnChallengeResultSubmitted
	//
	static public void SubmitChallengeResult(eChallengeResult result, string resultDescription)
	{
		PerformCommand(String.Format("SubmitChallengeResult|{0}|{1}", (int)result, resultDescription));
	}
	
	// DisplayChallengeCompletedModalWithPlayerPrefs //
	//
	// Displays the modal for showing the result of the challenge and allows for the user
	// to re-challenge (if it was a multi-attempt challenge). Call SubmitChallengeResult BEFORE
	// calling this.
	//	- result:					The result of the challenge.
	//	- resultDescription:		A string that will be prefixed by either the recipient's name
	//								or "You", depending on the player that is viewing the result.
	//								It should not contain whether the recipient won or lost, but
	//								should contain the statistics of his attempt. (Ex. "beat 30
	//								monsters" will turn into "You beat 30 monsters")
	//	- reChallengeDescription:	If the user decides to send out the result as a re-challenge,
	//								this will be the description for the new challenge. It should
	//								display what needs to be fulfilled to complete the challenge.
	//								Only used by multi-attempt challenges.
	//	- reChallengeDataKeys:		Tells the native code how/where to access the challenge data if
	//								the user decides to send out a "re-challenge". Only used for
	//								multi-attempt challenges.
	//
	static public void DisplayChallengeCompletedModalWithPlayerPrefs(eChallengeResult result,
		string resultDescription, string reChallengeDescription, string[] reChallengeDataKeys)
	{
		string commandString = String.Format("DisplayChallengeCompletedModalWithPlayerPrefs|{0}|{1}|{2}",
			(int)result, resultDescription, reChallengeDescription);
			
		if(reChallengeDataKeys != null)
		{
			for(int i = 0; i < reChallengeDataKeys.Length; i++)
			{
				commandString += "|" + reChallengeDataKeys[i];
			}
		}
		
		PerformCommand(commandString);
	}
	
	// DisplayChallengeCompletedModalWithFile //
	//
	// Displays the modal for showing the result of the challenge and allows for the user
	// to re-challenge (if it was a multi-attempt challenge). Call SubmitChallengeResult BEFORE
	// calling this.
	//	- result:					The result of the challenge.
	//	- resultDescription:		A string that will be prefixed by either the recipient's name
	//								or "You", depending on the player that is viewing the result.
	//								It should not contain whether the recipient won or lost, but
	//								should contain the statistics of his attempt. (Ex. "beat 30
	//								monsters" will turn into "You beat 30 monsters")
	//	- reChallengeDescription:	If the user decides to send out the result as a re-challenge,
	//								this will be the description for the new challenge. It should
	//								display what needs to be fulfilled to complete the challenge.
	//								Only used by multi-attempt challenges.
	//	- reChallengeDataFilePath:	Tells the native code how/where to access the challenge data if
	//								the user decides to send out a "re-challenge". Only used for
	//								multi-attempt challenges.
	//
	static public void DisplayChallengeCompletedModalWithFile(eChallengeResult result, string resultDescription,
		string reChallengeDescription, string reChallengeDataFilePath)
	{
		PerformCommand(String.Format("DisplayChallengeCompletedModalWithFile|{0}|{1}|{2}|{3}",
			(int)result, resultDescription, reChallengeDescription, reChallengeDataFilePath));
	}
	
	// GetChallengeDefinition //
	//
	// Get a challenge defintion structure for a particular challenge ID.
	//	- challengeId:	The ID of the challenge.
	//	- callback:		Called upon success or failure of the get. CANNOT be 'null'.
	//
	static public void GetChallengeDefinition(string challengeId, ChallengeDefinitionResultDelegate callback)
	{
		if(callback != null)
		{
			if(challengeDefinitionResultCallback == null)
			{
				challengeDefinitionResultCallback = callback;
				PerformCommand(String.Format("GetChallengeDefinition|{0}", challengeId));
			}
			else
			{
				Debug.LogError("OpenFeint error: Cannot call GetChallengeDefinition until previous call has returned success or failure.");
			}
		}
		else
		{
			Debug.LogError("OpenFeint error: No callback supplied to GetChallengeDefinition.");
		}
	}
	
	static public void GetChallengeDefinition(long challengeId, ChallengeDefinitionResultDelegate callback)
	{
		GetChallengeDefinition(challengeId.ToString(), callback);
	}
	
	// UploadBlobFromPlayerPrefs //
	//
	// Upload a data blob to the "cloud", which can be later retrieved on other devices.
	//	- blobKey:			The key of the blob you wish to upload to the cloud.
	//	- playerPrefsKeys:	An array of PlayerPrefs keys you wish to send to the cloud.
	//
	//	events				OnUploadedCloudBlob
	//
	static public void UploadBlobFromPlayerPrefs(string blobKey, string[] playerPrefsKeys)
	{
		string commandString = String.Format("UploadBlobFromPlayerPrefs|{0}", blobKey);
		
		for(int i = 0; i < playerPrefsKeys.Length; i++)
		{
			commandString += "|" + playerPrefsKeys[i];
		}
		
		PerformCommand(commandString);
	}
	
	// UploadBlobFromFile //
	//
	// Upload a data blob to the "cloud", which can be later retrieved on other devices.
	//	- blobKey:			The key of the blob you wish to upload to the cloud.
	//	- filePath:			A path to a file in the app's Documents directory that you wish to
	//						upload to the cloud. This is the preferred method if using
	//						serialized XML data (Unity iPhone Advanced only).
	//
	//	events				OnUploadedCloudBlob
	//
	static public void UploadBlobFromFile(string blobKey, string filePath)
	{
		PerformCommand(String.Format("UploadBlobFromFile|{0}|{1}", blobKey, filePath));
	}
	
	// DownloadBlobToPlayerPrefs //
	//
	// Download a previously uploaded data blob from the "cloud".
	//	- blobKey:			The key of the blob you wish to download from the cloud.
	//	- playerPrefsKeys:	An array of PlayerPrefs keys that will be populated with the
	//						downloaded blob data.
	//	- callback:			Called whenever the action returns success or failure.
	//						CANNOT be 'null'.
	//
	//	events				OnDownloadedCloudBlob
	//
	static public void DownloadBlobToPlayerPrefs(string blobKey, string[] playerPrefsKeys, DeferredResultDelegate callback)
	{
		if(callback != null)
		{
			if(downloadCloudBlobResultCallback == null)
			{
				downloadCloudBlobResultCallback = callback;
				
				string commandString = String.Format("DownloadBlobToPlayerPrefs|{0}", blobKey);
				for(int i = 0; i < playerPrefsKeys.Length; i++)
				{
					commandString += "|" + playerPrefsKeys[i];
				}
				
				PerformCommand(commandString);
			}
			else
			{
				Debug.LogError("OpenFeint error: Cannot call DownloadBlobToPlayerPrefsKeys until previous blob download has returned success or failure.");
			}
		}
		else
		{
			Debug.LogError("OpenFeint error: No callback supplied to DownloadBlobToPlayerPrefsKeys.");
		}
	}
	
	// DownloadBlobToFile //
	//
	// Download a previously uploaded data blob from the "cloud".
	//	- blobKey:			The key of the blob you wish to download from the cloud.
	//	- filePath:			A path to a file in the app's Documents directory that you wish
	//						to be populated with the downloaded blob contents. This is the
	//						preferred method if using serialized XML data (Unity iPhone
	//						Advanced only).
	//	- callback:			Called whenever the action returns success or failure.
	//						CANNOT be 'null'.
	//
	static public void DownloadBlobToFilePath(string blobKey, string filePath, DeferredResultDelegate callback)
	{
		if(callback != null)
		{
			if(downloadCloudBlobResultCallback == null)
			{
				downloadCloudBlobResultCallback = callback;
				
				PerformCommand(String.Format("DownloadBlobToFile|{0}|{1}", blobKey, filePath));
			}
			else
			{
				Debug.LogError("OpenFeint error: Cannot call DownloadBlobToFilePath until previous blob download has returned success or failure.");
			}
		}
		else
		{
			Debug.LogError("OpenFeint error: No callback supplied to DownloadBlobToFilePath.");
		}
	}
	
	// SetDashboardOrientation //
	//
	// Set the orientation of the OpenFeint Dashboard.
	//	- orientation:	The orientation to set to.
	//
	static public void SetDashboardOrientation(DeviceOrientation orientation)
	{
		PerformCommand(String.Format("SetDashboardOrientation|{0}", (int)orientation));
	}
	
	// PresentUserFeintApprovalModal //
	//
	// Present the OpenFeint user approval modal.
	//
	//	events		OnFeintApprovalScreenClosed
	//
	static public void PresentUserFeintApprovalModal()
	{
		PerformCommand("PresentUserFeintApprovalModal");
	}
	
	// IsLocalUserFollowingAnyone //
	//
	// Determine if the local user if following anyone.
	//	- callback:	Called upon receiving results from the call. Second parameter is set to whether
	//				or not the user is following anyone. CANNOT be 'null'.
	//
	static public void IsLocalUserFollowingAnyone(FollowingResultDelegate callback)
	{
		if(callback != null)
		{
			if(followingResultCallback == null)
			{
				followingResultCallback = callback;
				PerformCommand("IsLocalUserFollowingAnyone");
			}
			else
			{
				Debug.LogError("OpenFeint error: Cannot call IsLocalUserFollowingAnyone until previous call has returned success or failure.");
			}
		}
		else
		{
			Debug.LogError("OpenFeint error: No callback supplied to IsLocalUserFollowingAnyone.");
		}
	}
	
	static public void ClearCallbacks()
	{
		followingResultCallback = null;
		challengeDefinitionResultCallback = null;
		downloadCloudBlobResultCallback = null;
	}
	
	
	// PROPERTIES //
	// These properties allow you to get (and sometimes set) current state
	// information from OpenFeint.
	//
	// Example:
	//		if(OpenFeint.isOnline)
	//		{
	//			Debug.Log("User is online!");
	//		}
	//
	
	// Check whether or not the local user has approved use of OpenFeint.
	static public bool hasUserApprovedFeint
	{
		get
		{
			PerformCommand("HasUserApprovedFeint");
			return BoolResult;
		}
	}
	
	// Check whether or not the local user is online.
	static public bool isOnline
	{
		get
		{
			PerformCommand("IsOnline");
			return BoolResult;
		}
	}
	
	// Get the user name of the local user.
	static public string localUserName
	{
		get
		{
			PerformCommand("LocalUserName");
			return StringResult;
		}
	}
	
	// Get the user ID of the local user.
	static public string localUserID
	{
		get
		{
			PerformCommand("LocalUserID");
			return StringResult;
		}
	}
	
	// Set/get whether or not we are allowing OpenFeint notifications.
	static public bool allowNotifications
	{
		get		{ return ((PlayerPrefs.GetInt("OFAllowNotifications") == 1) ? true : false); }
		set		{ PlayerPrefs.SetInt("OFAllowNotifications", ((value) ? 1 : 0)); }
	}
	
	// The number of unviewed challenges when the user logged in.
	static public int numberOfUnviewedChallengesAtLogin
	{
		get		{ return PlayerPrefs.GetInt("OFBootedWithUnviewedChallenges"); }
	}
	
	
	// PRIVATE //
	// The following is used by the public methods and properties above. They
	// should not be used directly.
	
	// Sends a command to the Objective-C command listener.
	static private void PerformCommand(string cmd)
	{
		if(!Application.isEditor)
		{
			PlayerPrefs.SetString("OFCommand", cmd);
		}
		else
		{
			Debug.Log("OpenFeint: " + cmd);
		}
	}
	
	// Results from performing a command.
	static private bool BoolResult		{ get { return ((PlayerPrefs.GetInt("OFResult") == 1) ? true : false); } }
	static private int IntResult		{ get { return PlayerPrefs.GetInt("OFResult"); } }
	static private float FloatResult	{ get { return PlayerPrefs.GetFloat("OFResult"); } }
	static private string StringResult	{ get { return PlayerPrefs.GetString("OFResult"); } }
	static private bool HasResult		{ get { return PlayerPrefs.HasKey("OFResult"); } }
	
	// Methods called by Objective-C to trigger script events.
	static private void TriggerGeneralEvent(int eventIndex)
	{
		switch(eventIndex)
		{
			case 0:	if(OnRestartedChallenge != null)				OnRestartedChallenge();				break;
			case 1:	if(OnRestartedCreateChallenge != null)			OnRestartedCreateChallenge();		break;
			case 2:	if(OnCompletedChallengeScreenClosed != null)	OnCompletedChallengeScreenClosed();	break;
			case 3: if(OnSendChallengeScreenClosed != null)			OnSendChallengeScreenClosed();		break;
			case 4:	if(OnUserSentChallenges != null)				OnUserSentChallenges();				break;
		}
	}
	
	static private void TriggerDeferredResultEvent(int eventIndex, bool result)
	{
		switch(eventIndex)
		{
			case 0:	if(OnHighScoreSubmitted != null)		OnHighScoreSubmitted(result);		break;
			case 1:	if(OnAchievementUnlocked != null)		OnAchievementUnlocked(result);		break;
			case 2:	if(OnChallengeResultSubmitted != null)	OnChallengeResultSubmitted(result);	break;
			case 3:	if(OnUploadedCloudBlob != null)			OnUploadedCloudBlob(result);		break;
		}
	}
	
	static private void TriggerNotificationEvent(string text, int category, int type)
	{
		if(OnNotification != null)
		{
			OnNotification(text, (eNotificationCategory)category, (eNotificationType)type);
		}
	}
	
	static private void TriggerUserLogInEvent(int eventIndex, string userId)
	{
		switch(eventIndex)
		{
			case 0:	if(OnUserLoggedIn != null)	OnUserLoggedIn(userId);		break;
			case 1:	if(OnUserLoggedOut != null)	OnUserLoggedOut(userId);	break;
		}
	}
	
	static private void TriggerStartedChallengeEvent(string title, long id, string iconUrl, bool multiAttempt)
	{
		if(OnStartedChallenge != null)
		{
			ChallengeDefinition definition;
			definition.title = title;
			definition.challengeId = id;
			definition.iconUrl = iconUrl;
			definition.multiAttempt = multiAttempt;
			
			OnStartedChallenge(definition);
		}
	}
	
	// Methods called from Objective-C to trigger callbacks.
	static private void TriggerFollowingResultCallback(bool success, bool result)
	{
		if(followingResultCallback != null)
		{
			followingResultCallback(success, result);
			followingResultCallback = null;
		}
	}
	
	static private void TriggerChallengeDefinitionResultCallback(bool success)
	{
		if(challengeDefinitionResultCallback != null)
		{
			ChallengeDefinition definition;
			definition.title = PlayerPrefs.GetString("OFChallengeDefinition_title");
			definition.challengeId = PlayerPrefs.GetInt("OFChallengeDefinition_challengeId");
			definition.iconUrl = PlayerPrefs.GetString("OFChallengeDefinition_iconUrl");
			definition.multiAttempt = ((PlayerPrefs.GetInt("OFChallengeDefinition_multiAttempt") == 1) ? true : false);
			
			challengeDefinitionResultCallback(success, definition);
			challengeDefinitionResultCallback = null;
		}
	}
	
	static private void TriggerDownloadCloudBlobResultCallback(bool success)
	{
		if(downloadCloudBlobResultCallback != null)
		{
			downloadCloudBlobResultCallback(success);
		}
	}
	
	// Stored callbacks.
	static private FollowingResultDelegate followingResultCallback;
	static private ChallengeDefinitionResultDelegate challengeDefinitionResultCallback;
	static private DeferredResultDelegate downloadCloudBlobResultCallback;
}
